/*
  A class that represents one PWM output. 

  by Axel Brinkeby 
  2018-12-21
*/


#include "Arduino.h"
#include "Pwmoutput.h"

Pwmoutput::Pwmoutput() {  
  _value = 0; 
  _minLimit = 1100;
  _maxLimit = 1900;
  resetAllInputs();
}

void Pwmoutput::attachOnPin(int pin) {
  _pwmOutput.attach(pin);
}

void Pwmoutput::setInput(int index, float amount) {
  amount = constrain(amount, -1.0, 1.0);
  if (index < 15)
    _inputChannelMix[index] = amount;
  else
    _inputGyroMix[index - 16] = amount;
}

void Pwmoutput::resetAllInputs() {
  for (int i = 0; i < 16; i++) {
    _inputChannelMix[i] = 0; 
  }
}

// "channels" is raw SBUS channel data, ranging from 0 to 2000 per channel, 16 channels
// "gyroData" is three float values containing gyro data for each axis, unit is degrees per second
void Pwmoutput::setInputValues(uint16_t channels[], float gyroData[]) {
  _value = 0;  
   
  for (int i = 0; i < 16; i++) {
    float normalizedChannelValue = ((channels[i] - 1000.0) / 1000.0);
    _value += _inputChannelMix[i] * normalizedChannelValue;
  }
  
  for (int i = 0; i < 3; i++) {
    //normalizedGyroValue equals 1 means rotating one revulution per second
    float normalizedGyroValue = (gyroData[i] / 360.0); 
    _value += _inputGyroMix[i] * normalizedGyroValue;
  }

  _value = constrain(_value, -1.0, 1.0); 
} 

void Pwmoutput::updateOutput() {
  uint16_t outValue = (uint16_t)( 1000.0 + ((_value + 1.0) * 500.0) );
  constrain(outValue, _minLimit, _maxLimit);  
  _pwmOutput.writeMicroseconds(outValue);
}

void Pwmoutput::setMin(int value){
  _minLimit = value;
}    
    
void Pwmoutput::setMax(int value){
  _maxLimit = value;
}

int Pwmoutput::getMin() {
  return _minLimit; 
}

int Pwmoutput::getMax() {
  return _maxLimit; 
}

